/*
 * Hanashizume no Matsuri for X
 * Copyright (c) 2000 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: font.c,v 1.1 2000/12/18 16:57:32 tf Exp $
 */

#include <X11/Xlib.h>

#include "hana.h"

void
InitFont(Display *disp, GC gc, XFontStruct *f)
{
  if (f != NULL && f->fid != None) {
    XSetFont(disp, gc, f->fid);
  } else {
    fprintf(stderr, "Can't get the font.\n");
    exit(1);
  }
  finfo.disp = disp;
  finfo.gc = gc;
  finfo.xfont = f;
  finfo.width = f->max_bounds.rbearing + XPAD;
  finfo.top = f->max_bounds.ascent;
  finfo.pitch = f->max_bounds.ascent + f->max_bounds.descent + YPAD;
}

void
PutChar(Drawable dst, int *code, int *x, int *y, long foreground, long shadow, int attr)
{
  XChar2b ch;
  int yy;
  
  ch = u2xch(*code);

  yy = *y + finfo.top;
  
  XSetForeground(finfo.disp, finfo.gc, shadow);
  switch (attr) {
  case CH_SHADOW:
    XDrawString16(finfo.disp, dst, finfo.gc, *x - 1, yy    , &ch, 1);
    XDrawString16(finfo.disp, dst, finfo.gc, *x    , yy - 1, &ch, 1);

    XDrawString16(finfo.disp, dst, finfo.gc, *x + 1, yy    , &ch, 1);
    XDrawString16(finfo.disp, dst, finfo.gc, *x    , yy + 1, &ch, 1);
    
    XDrawString16(finfo.disp, dst, finfo.gc, *x + 1, yy + 1, &ch, 1);
    XDrawString16(finfo.disp, dst, finfo.gc, *x + 2, yy + 2, &ch, 1);
    break;
  case CH_BORDER:
    XDrawString16(finfo.disp, dst, finfo.gc, *x - 1, yy    , &ch, 1);
    XDrawString16(finfo.disp, dst, finfo.gc, *x    , yy - 1, &ch, 1);

    XDrawString16(finfo.disp, dst, finfo.gc, *x + 1, yy    , &ch, 1);
    XDrawString16(finfo.disp, dst, finfo.gc, *x    , yy + 1, &ch, 1);
    break;
  }
  
  XSetForeground(finfo.disp, finfo.gc, foreground);
  XDrawString16(finfo.disp, dst, finfo.gc, *x, yy, &ch, 1);

  *x += finfo.width;
}
