/*
 * Hanashizume no Matsuri for X
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: vidmode.c,v 1.4 2001/04/19 15:47:09 tf Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include "hana.h"
#include "vidmode.h"

static void restoreMode(void);
static XF86VidModeModeInfo *checkModeLinesRes(int, int, int);
static void ToScreenMode(Widget);

XF86VidModeModeInfo *
SetupVidMode(Widget top, Bool screen_mode)
{
  int major, minor;
  int eventBase, errorBase;
  XF86VidModeModeInfo *sel = NULL;
  Display *d;

  fprintf(stderr, "Setting up the video mode...");

  /* initialize values */
  vm.enable_screen_mode = False;
  vm.screen_mode = False;
  d = XtDisplay(top);
  
  if (!XF86VidModeQueryVersion(d, &major, &minor)) {
    vm.enable_screen_mode = False;
    fprintf(stderr, "failed (Can't get the extension version).\n");
    return NULL;
  } else if (!XF86VidModeQueryExtension(d, &eventBase, &errorBase)) {
    vm.enable_screen_mode = False;
    fprintf(stderr, "failed (Can't get the extension info.).\n");
    return NULL;
  } else if (major < MINMAJOR || (major == MINMAJOR && minor < MINMINOR)) {
    vm.enable_screen_mode = False;
    fprintf(stderr, "failed (VidModeExtension (%d.%d) is older than required version (%d.%d)).\n", major, minor, MINMAJOR, MINMINOR);
    return NULL;
  } else {
    if (!XF86VidModeGetAllModeLines(d, XScreenNumberOfScreen(XtScreen(top)),
				    &(vm.vid_count), &(vm.modeinfos))) {
      fprintf(stderr, "Can't get modeinfos.\n");
      vm.enable_screen_mode = False;
    } else if ((sel = checkModeLinesRes(vm.vid_count, 640, 480)) == NULL &&
	       (sel = checkModeLinesRes(vm.vid_count, 800, 600)) == NULL) {
      fprintf(stderr, "Good mode isn't found...\n");
      vm.enable_screen_mode = False;
    } else {
      vm.enable_screen_mode = True;
      vm.dpy_restore = d;
      vm.scn_restore = XScreenNumberOfScreen(XtScreen(top));
      atexit(restoreMode);
    }
  }

  /* change video mode */
  if (vm.enable_screen_mode && screen_mode) {
    ToScreenMode(top);
    vm.screen_mode = True;
  }

  fprintf(stderr, "done.\n");
  return sel;
}

static void
restoreMode(void)
{
  if (vm.screen_mode) {
    XF86VidModeSwitchToMode(vm.dpy_restore, vm.scn_restore, vm.modeinfos[0]);
    XSync(vm.dpy_restore, False);
    vm.screen_mode = False;
  }
}

static XF86VidModeModeInfo *
checkModeLinesRes(int count, int w, int h)
{
  int i;

  for (i = 0; i < count; i++) { 
    if (vm.modeinfos[i]->hdisplay == w && vm.modeinfos[i]->vdisplay == h)  
      return vm.modeinfos[i];
  }
  return NULL;
}

static void
ToScreenMode(Widget top)
{
  XF86VidModeModeInfo  *sel;

  sel = checkModeLinesRes(vm.vid_count, 640, 480);
  if (sel == NULL &&
      (sel = checkModeLinesRes(vm.vid_count, 800, 600)) == NULL) {
    fprintf(stderr, "WARNING: Good mode is not found...\n");
    vm.enable_screen_mode = False;
  } else if (!XF86VidModeSwitchToMode(XtDisplay(top), XScreenNumberOfScreen(XtScreen(top)), sel)) {
    fprintf(stderr, "WARNING: Can't set modeline.\n");
    vm.enable_screen_mode = False;
  } else {   
    XF86VidModeSetViewPort(XtDisplay(top),
			   XScreenNumberOfScreen(XtScreen(top)),
			   0, 0);
    XWarpPointer(XtDisplay(top),
		 None, 
		 RootWindowOfScreen(XtScreen(top)),
		 0, 0, 0, 0, 0, 0);
    XSync(XtDisplay(top), False);
    XtVaSetValues(top, XtNoverrideRedirect, True, NULL);
  /* in fact, this function should not be used... */
    XtConfigureWidget(top, 0, 0, sel->hdisplay, sel->vdisplay, 0);
    XtAddEventHandler(top, EnterWindowMask, True, (XtEventHandler)SetFocus, NULL);
    vm.screen_mode = True;
  }   
}

void
ScreenWindow(Widget w, XEvent *e, String *p, Cardinal *n)
{
  if (vm.screen_mode) {
    DisableScreen();
  } else {
    EnableScreen();
  }
}

void
EnableScreen()
{
  if (!vm.enable_screen_mode || vm.screen_mode) {
    return;
  }

  XtUnmapWidget(top);
  ToScreenMode(top);
  vm.screen_mode = True;

  XtMapWidget(top);
  XGrabPointer (XtDisplay(top), 
		XtWindow(top), True, 0,
		GrabModeAsync, GrabModeAsync, 
		XtWindow(top), None, CurrentTime);
  
  XMoveWindow(XtDisplay(top), XtWindow(top), 0, 0);
  XWarpPointer(XtDisplay(top),
	       None, 
	       RootWindowOfScreen(XtScreen(top)),
	       0, 0, 0, 0, 0, 0);
  XSync(XtDisplay(top), False);
}

void
DisableScreen()
{
  int width  = 640;
  int height = 480;

  if (!vm.enable_screen_mode || !vm.screen_mode) {
    return;
  }

  XtUnmapWidget(top);
  restoreMode();
  vm.screen_mode = False;
  XtRemoveEventHandler(top, EnterWindowMask, True, (XtEventHandler)SetFocus, NULL);
  XtVaSetValues(top, 
		XtNwidth, width,
		XtNheight, height,
		XtNoverrideRedirect, False, NULL);
  XtMapWidget(top);
}
